


#include "nommage.hh"
#include "bool.hh"


//
//   Et bien sur l'implantation des deux 
//   fonctions import et export. Lisez, les
//   c'est interessant de voir comment fonctionne
//   le service de nom
//


CORBA::Object_ptr import(CORBA::ORB_ptr orb, CosNaming::Name nom)
{

	CosNaming::NamingContext_var racine;
  
  try 	{
	// On commence par se connecter au serveur de nom
	// en obtenant une reference
	CORBA::Object_var initServ;
	initServ = orb->resolve_initial_references("NameService");

	// "cast" dynamique de la reference en
	// contexte
	racine = CosNaming::NamingContext::_narrow(initServ);
	if (CORBA::is_nil(racine)) 
		{
		cerr<<"import : Echec lors du narrow  (service de nom) "<<endl;
		return CORBA::Object::_nil();
		}

       	}

  catch(CORBA::ORB::InvalidName& ex) {
	cerr << "import : Echec connection omniNames (service de nom) "<<endl;
	return CORBA::Object::_nil();
	}




	// Bien, on peut maintenant resoudre
	// le nom et obtenir l'IOR de l'objet serveur
	CORBA::Object_ptr obj;

  try 	{
	obj = racine->resolve(nom);
      	}

  catch(CosNaming::NamingContext::NotFound& ex)
    	{
      	// Le contexte n'a pas ete trouve !
      	cerr << "import : Contexte non trouve "
         	<< "(service de nom)" << endl;
      	return CORBA::Object::_nil();
    	}
  catch (CORBA::COMM_FAILURE& ex) 
	{
    	cerr << "import : Exception systeme  COMM_FAILURE"
         	<< "(service de nom)" << endl;
    	return CORBA::Object::_nil();
  	}
  catch(omniORB::fatalException& ex) 
	{
	throw;
  	}
  catch (...) 
	{
    	cerr << "import : Exception systeme levee"
         	<< "(service de nom)" << endl;
    	return CORBA::Object::_nil();
  	}


  	return obj;
}




//
//  Permet d'ajouter au serveur de nom un contexe et un nom d'objet 
//
CORBA::Boolean export(CORBA::ORB_ptr orb, CosNaming::Name contexte, CosNaming::Name objet,
			CORBA::Object_ptr obj)
{

CosNaming::NamingContext_var racine;
  
  try 	{
	// On commence par se connecter au serveur de nom
	// en obtenant une reference
	CORBA::Object_var initServ;
	initServ = orb->resolve_initial_references("NameService");

	// "cast" dynamique de la reference en
	// contexte
	racine = CosNaming::NamingContext::_narrow(initServ);
	if (CORBA::is_nil(racine)) 
		{
		cerr<<"export : Echec lors du narrow  (service de nom) "<<endl;
		return FALSE;
		}

       	}

  catch(CORBA::ORB::InvalidName& ex) {
	cerr << "export : Echec connection omniNames (service de nom) "<<endl;
	return FALSE;
	}


//
// Maintenant, on ajoute l'objet dans
// l'arborescence du service de nom
//

CosNaming::NamingContext_var ctx;

try 
  {


   try 
	{
      	// Liaison du context a la racine de l'arborescence
	// du service de nom
      	ctx = racine->bind_new_context(contexte);
	}

   catch(CosNaming::NamingContext::AlreadyBound& ex) 
	{
      	// Si le contexte existe deja, cette exception est levee
      	// on se contente alors de resoudre le nom et d'initialiser
	// convenablement ctx
      	// 
      	CORBA::Object_var obj = racine->resolve(contexte);
      	ctx = CosNaming::NamingContext::_narrow(obj);
      	if (CORBA::is_nil(ctx)) 
		{
        	cerr << "export : Echec sur le narrow (service de nom)" << endl;
        	return FALSE;
      		}
    	} 


	// 
	// Meme traitement pour l'objet
	//
   try 
	{
      	ctx->bind(objet,obj);
    	}

   catch(CosNaming::NamingContext::AlreadyBound& ex) 
	{
      	ctx->rebind(objet,obj);
    	}

  
}


  catch (CORBA::COMM_FAILURE& ex) 
	{
    	cerr << "export : Exception systeme COMM_FAILURE  "
        		 << " (service de nom)" << endl;
    	return FALSE;
  	}
  catch (omniORB::fatalException& ex) 
	{
    	throw;
  	}
  catch (...) 
	{
    	cerr << "export : Exception systeme (service de nom)"<<endl;
    	return FALSE;
  	}


  return TRUE;
}



